//-----------------------------------------------------------------------------
//     The confidential and proprietary information contained in this file may
//     only be used by a person authorised under and to the extent permitted
//     by a subsisting licensing agreement from ARM Limited or its affiliates.
//
//            (C) COPYRIGHT 2015-2019 ARM Limited or its affiliates.
//                ALL RIGHTS RESERVED
//
//     This entire notice must be reproduced on all copies of this file
//     and copies of this file may only be made by a person if such person is
//     permitted to do so under the terms of a subsisting license agreement
//     from ARM Limited or its affiliates.
//
//     Release Information : ENYO-MP096-r1p0-00eac0
//
//-----------------------------------------------------------------------------
// SystemVerilog (IEEE Std 1800-2012)
//-----------------------------------------------------------------------------




`include "enyo_header.sv"


module enyo_vx_sha256su1
(


  input wire            sha256su1_x_op,
  input wire            sha256su1_y_op,
  input wire            sha256su1_z_op,
  input wire [63:0]     x,
  input wire [63:0]     y,
  input wire [63:0]     z,
  output wire [63:0]   newx 
);







  wire [63:0]                            x_nop;
  wire [63:0]                            y_nop;
  wire [63:0]                            z_nop;
  wire [63:0]                            z_rot_v2;
  wire [63:0]                            zror17_v2;
  wire [63:0]                            zror19_v2;
  wire [63:0]                            zshr10_v2;




assign x_nop[63:0] = x[63:0] &  {64{sha256su1_x_op}};
assign y_nop[63:0] = y[63:0] &  {64{sha256su1_y_op}};
assign z_nop[63:0] = z[63:0] &  {64{sha256su1_z_op}};

assign zror17_v2[63:0] = {z_nop[48:32],  z_nop[63:49],
                          z_nop[16:0],   z_nop[31:17]};
assign zror19_v2[63:0] = {z_nop[50:32],  z_nop[63:51],
                          z_nop[18:0],   z_nop[31:19]};
assign zshr10_v2[63:0] = {10'b00_0000_0000, z_nop[63:42],
                          10'b00_0000_0000, z_nop[31:10]};
assign z_rot_v2[63:0] = zror17_v2[63:0] ^ zror19_v2[63:0] ^ zshr10_v2[63:0];

assign newx[63:32] = x_nop[63:32] + y_nop[63:32] + z_rot_v2[63:32];
assign newx[31:0]  = x_nop[31:0]  + y_nop[31:0]  + z_rot_v2[31:0];

endmodule


`define ENYO_UNDEFINE
`include "enyo_header.sv"
`undef ENYO_UNDEFINE
